package platform_user

import (
	"context"
	"github.com/robfig/cron/v3"
	"github.com/zeromicro/go-zero/core/logx"
	pbMq "live-service/app/pb/mq"
	"live-service/app/user_center/model"
	"live-service/app/user_center/rpc/internal/svc"
	"time"
)

// PlatformUser 仅提取需要的部分,其余的json后装到p_info
type (
	PlatformUser struct {
		PUid    string
		PName   string
		PAvatar string
		PInfo   string
	}
	UserRetriever struct {
		ctx    context.Context
		svcCtx *svc.ServiceContext

		retrieveMapper map[string]RetrieveFunc

		logx.Logger
	}
)

type RetrieveFunc func(pUid string) (*PlatformUser, error)

func NewUserRetriever(svcCtx *svc.ServiceContext) *UserRetriever {
	ctx := context.Background()
	result := &UserRetriever{
		ctx:    ctx,
		svcCtx: svcCtx,
		retrieveMapper: map[string]RetrieveFunc{
			pbMq.Platform_name[int32(pbMq.Platform_bilibili)]: RetrieveBilibili,
		},
		Logger: logx.WithContext(ctx),
	}
	return result
}

func (r *UserRetriever) retrieveUser(platform, pUid string) *PlatformUser {
	if handle, ok := r.retrieveMapper[platform]; ok {
		if p, err := handle(pUid); err != nil {
			r.Logger.Errorf("获取平台用户信息错误: err: %s\n", err.Error())
		} else {
			return p
		}
	}
	return &PlatformUser{
		PUid:    pUid,
		PName:   "",
		PAvatar: "",
		PInfo:   "{}", // 空json
	}
}

func (r *UserRetriever) Scheduler() {
	logx.Info("开启用户数据采集服务...")
	c := cron.New()
	_, _ = c.AddFunc("@every 1s", func() {
		var list []model.UserPlatform
		var err error
		if list, err = r.svcCtx.UserPlatformModel.FindEmptyList(r.ctx, 5); err != nil {
			r.Logger.Errorf("db err", err)
			return
		}
		r.retrieveList(list)
		// 优先加载空信息用户
		if len(list) <= 0 {
			if list, err = r.svcCtx.UserPlatformModel.FindUpdatableList(r.ctx, r.svcCtx.Config.User.UpdateDuration, 5); err != nil {
				return
			}
			r.retrieveList(list)
		}
	})
	c.Start()
}

func (r *UserRetriever) retrieveList(list []model.UserPlatform) {
	for _, dbModel := range list {
		pUser := r.retrieveUser(dbModel.Platform, dbModel.PUid)
		if pUser.PInfo == "{}" {
			continue
		}
		dbModel.PUname = pUser.PName
		dbModel.PAvatar = pUser.PAvatar
		dbModel.PInfo = pUser.PInfo
		dbModel.UpdateTime = time.Now()
		_ = r.svcCtx.UserPlatformModel.Update(r.ctx, &dbModel)
	}
}
